#ifndef BASE_BASE_H
#define BASE_BASE_H

#include <vector>

#include "math/vec.h"	//for line3f - for the mouseline
#include "viewport.h"	//mouseOverViewport
#include "view.h"
#include "console.h"
#include "cvar.h"

#define MULTIVIEW

enum {
#ifdef MULTIVIEW
	VIEW_XY,
	VIEW_XZ,
	VIEW_YZ,
#endif
	VIEW_3D,
	VIEW_COUNT
};

enum {
	VIEWPORT_FULL,
#ifdef MULTIVIEW
	VIEWPORT_UR,
	VIEWPORT_UL,
	VIEWPORT_LR,
	VIEWPORT_LL,
#endif
	VIEWPORT_COUNT
};

class LoopCallback {
public:
	virtual void update() = 0;
};

#define _VIEW_INIT_DIST	15
#define _ORTHO_SIZE		15
#define _VIEW_ZNEAR		1.f
#define _VIEW_ZFAR		5000.f

class Base {
public:
	Base() {reset();}
	~Base() {shutdown();}

	void init();
	void shutdown();

	void loop(void (*loopCallback)());
	void loop(LoopCallback *loopCallback);
	void doLoop();

	void setScene(Scene *scene);

	//if you think putting this here is a mess,
	//you should see the C++ code...
	//here's the mouseline from the view into the scene, and the viewport it lies over
	line3f mouseLine;
	Viewport *mouseOverViewport;

	//here's our current camera speed and rotation:
	float camSpeed;
	float camRot;
	float camZoom;

	//whether we allow the camera key overrides
	bool useCameraKeys;
	
	//whether we allow the user to use the console or not.
	bool useConsole;

	//enum of all the views
	View views[VIEW_COUNT];
	
	//and of all viewports
	//TODO - make these dynamic structures so the app has control over them
	
	Viewport viewports[VIEWPORT_COUNT];
	
	/**
	 * set this variable in order to only update one viewport
	 * if this variable is NULL then all viewports are updated
	 * after display is called, the variable is reset to NULL
	 */
	//static Viewport *redisplayViewport = NULL;
	std::vector<Viewport *> activeViewports;

	//hack for the multiview system
	//still getting patted down...
	void clearAllViewports();
	
	//if (fullscreen) then only VIEWPORT_FULL, if (!fullscreen) then add all else...
	void restoreViewports();

	//find the viewport clicked upon
	Viewport *getTopViewAt(const vec2i &v);
	
	//the current input system is a bit of a mess ...
	Viewport *overrideInputViewport;

protected:
	void reset();
	void initViewports();
	void initViews();
	
	//sketchy
	//of course the whole screen swapping interface 
	//combined with abstracted view/viewports
	//is awfully sketchy
	bool isFullscreen();

	void (*loopCallback)();
	LoopCallback *loopCallbackObj;
};

extern Base base;
extern Console con;

#endif	//BASE_BASE_H